@@ -31,6 +31,7 @@
 
 #include "swfmoz_loader.h"
 #include "swfmoz_player.h"
+#include "plugin.h"
 #include "plugin_x11.h"
 
 NPNetscapeFuncs mozilla_funcs;
@@ -97,7 +98,7 @@ plugin_pop_allow_popups (NPP instance)
 char *
 NP_GetMIMEDescription (void)
 {
-  return "application/x-shockwave-flash:swf:Adobe Flash movie;application/futuresplash:spl:FutureSplash movie";
+  return (char *) "application/x-shockwave-flash:swf:Adobe Flash movie;application/futuresplash:spl:FutureSplash movie";
 }
 
 NPError 
@@ -112,15 +113,26 @@ NP_GetValue (void* reserved, NPPVariable var, void* out)
 		      
   switch (var) {
     case NPPVpluginNameString:
-      *val = "Shockwave Flash";
+      *val = (char *) "Shockwave Flash";
       break;
     case NPPVpluginDescriptionString:
       /* FIXME: find a way to encode the Swfdec version without breaking stupid JS scripts */
-      *val = "Shockwave Flash 9.0 r100";
+      *val = (char *) "Shockwave Flash 9.0 r100";
       break;
     case NPPVpluginNeedsXEmbed:
       *((PRBool*) val) = PR_TRUE;
       break;
+    case NPPVpluginWindowBool:
+    case NPPVpluginTransparentBool:
+    case NPPVjavaClass:
+    case NPPVpluginWindowSize:
+    case NPPVpluginTimerInterval:
+    case NPPVpluginScriptableInstance:
+    case NPPVpluginScriptableIID:
+    case NPPVjavascriptPushCallerBool:
+    case NPPVpluginKeepLibraryInMemory:
+    case NPPVpluginScriptableNPObject:
+    case NPPVformValue:
     default:
       return NPERR_INVALID_PARAM;
       break;
@@ -128,6 +140,7 @@ NP_GetValue (void* reserved, NPPVariable var, void* out)
   return NPERR_NO_ERROR;
 }
 
+G_MODULE_EXPORT gboolean swfdec_mozilla_make_sure_this_thing_stays_in_memory (void);
 /* This mess is unfortunately necessary */
 #define PLUGIN_FILE PLUGIN_DIR G_DIR_SEPARATOR_S "libswfdecmozilla." G_MODULE_SUFFIX
 G_MODULE_EXPORT gboolean
@@ -161,6 +174,7 @@ plugin_new (NPMIMEType mime_type, NPP instance,
     uint16_t mode, int16_t argc, char *argn[], char *argv[],
     NPSavedData * saved)
 {
+  SwfdecPlayer *player;
   int i;
 
   if (instance == NULL)
@@ -188,7 +202,7 @@ plugin_new (NPMIMEType mime_type, NPP instance,
   /* Init functioncalling (even g_type_init) gets postponed until we know we
    * won't be unloaded, i.e. NPPVpluginKeepLibraryInMemory was successful */
   swfdec_init ();
-  instance->pdata = swfmoz_player_new (instance, FALSE);
+  instance->pdata = player = swfmoz_player_new (instance, FALSE);
 
   /* set the properties we support */
   /* FIXME: figure out how variables override each other */
@@ -197,11 +211,11 @@ plugin_new (NPMIMEType mime_type, NPP instance,
       continue;
     if (g_ascii_strcasecmp (argn[i], "flashvars") == 0) {
       if (argv[i])
-	swfmoz_player_add_variables (instance->pdata, argv[i]);
+	swfdec_player_set_variables (player, argv[i]);
     } else if (g_ascii_strcasecmp (argn[i], "bgcolor") == 0) {
       GdkColor color;
       if (gdk_color_parse (argv[i], &color)) {
-	swfdec_player_set_background_color (SWFMOZ_PLAYER (instance->pdata)->player, 
+	swfdec_player_set_background_color (player, 
 	    0xFF000000 | (color.red / 0x101 << 16) | 
 	    (color.green / 0x101 << 8) | (color.blue / 0x101));
       }
@@ -220,7 +234,7 @@ plugin_new (NPMIMEType mime_type, NPP instance,
       } else {
 	scale = SWFDEC_SCALE_SHOW_ALL;
       }
-      swfdec_player_set_scale_mode (SWFMOZ_PLAYER (instance->pdata)->player, scale);
+      swfdec_player_set_scale_mode (player, scale);
     } else if (g_ascii_strcasecmp (argn[i], "salign") == 0) {
       struct {
 	const char *	name;
@@ -236,15 +250,15 @@ plugin_new (NPMIMEType mime_type, NPP instance,
 	{ "br", SWFDEC_ALIGNMENT_BOTTOM_RIGHT }
       };
       SwfdecAlignment align = SWFDEC_ALIGNMENT_CENTER;
-      guint i;
+      guint j;
 
-      for (i = 0; i < G_N_ELEMENTS (possibilities); i++) {
-	if (g_ascii_strcasecmp (argv[i], possibilities[i].name) == 0) {
-	  align = possibilities[i].align;
+      for (j = 0; j < G_N_ELEMENTS (possibilities); j++) {
+	if (g_ascii_strcasecmp (argv[j], possibilities[j].name) == 0) {
+	  align = possibilities[j].align;
 	  break;
 	}
       }
-      swfdec_player_set_alignment (SWFMOZ_PLAYER (instance->pdata)->player, align);
+      swfdec_player_set_alignment (player, align);
     } else {
       g_printerr ("Unsupported movie property %s with value \"%s\"\n", 
 	  argn[i], argv[i] ? argv[i] : "(null)");
@@ -254,7 +268,7 @@ plugin_new (NPMIMEType mime_type, NPP instance,
   return NPERR_NO_ERROR;
 }
 
-NPError 
+static NPError 
 plugin_destroy (NPP instance, NPSavedData **save)
 {
   if (instance == NULL || !SWFMOZ_IS_PLAYER (instance->pdata))
@@ -270,16 +284,15 @@ static NPError
 plugin_new_stream (NPP instance, NPMIMEType type, NPStream* stream, 
     NPBool seekable, uint16* stype)
 {
-  SwfdecLoader *loader;
-
   if (instance == NULL || !SWFMOZ_IS_PLAYER (instance->pdata))
     return NPERR_INVALID_INSTANCE_ERROR;
 
-  loader = swfmoz_player_add_stream (instance->pdata, stream);
-  if (loader == NULL)
-    return NPERR_INVALID_URL;
-  g_object_ref (loader);
-  stream->pdata = loader;
+  if (!SWFMOZ_IS_LOADER (stream->notifyData)) {
+    if (!swfmoz_player_set_initial_stream (instance->pdata, stream))
+      return NPERR_INVALID_URL;
+  } else {
+    swfmoz_loader_set_stream (stream->notifyData, stream);
+  }
   if (stype)
     *stype = NP_ASFILE;
   return NPERR_NO_ERROR;
@@ -294,7 +307,7 @@ plugin_destroy_stream_cb (NPP instance, NPStream* stream, NPReason reason)
     return NPERR_INVALID_INSTANCE_ERROR;
 
   swfmoz_loader_ensure_open (stream->pdata);
-  swfdec_loader_eof (stream->pdata);
+  swfdec_stream_eof (stream->pdata);
   SWFMOZ_LOADER (stream->pdata)->stream = NULL;
   g_object_unref (stream->pdata);
   return NPERR_NO_ERROR;
@@ -321,11 +334,9 @@ plugin_write (NPP instance, NPStream *stream, int32 offset, int32 len, void *buf
   if (!SWFMOZ_IS_LOADER (stream->pdata))
     return -1;
 
-  new = swfdec_buffer_new ();
-  new->length = len;
-  new->data = g_memdup (buffer, len);
+  new = swfdec_buffer_new_for_data (g_memdup (buffer, len), len);
   swfmoz_loader_ensure_open (stream->pdata);
-  swfdec_loader_push (stream->pdata, new);
+  swfdec_stream_push (stream->pdata, new);
   return len;
 }
 
@@ -372,14 +383,14 @@ plugin_handle_event (NPP instance, void *eventp)
 static void
 plugin_url_notify (NPP instance, const char* url, NPReason reason, void* notifyData)
 {
-  SwfdecLoader *loader = SWFDEC_LOADER (notifyData);
+  SwfdecStream *stream = SWFDEC_STREAM (notifyData);
 
   if (reason == NPRES_NETWORK_ERR) {
-    swfdec_loader_error (loader, "Network error");
+    swfdec_stream_error (stream, "Network error");
   } else if (reason == NPRES_USER_BREAK) {
-    swfdec_loader_error (loader, "User interrupt");
+    swfdec_stream_error (stream, "User interrupt");
   }
-  g_object_unref (loader);
+  g_object_unref (stream);
 }
 
 NPError
