@@ -22,6 +22,7 @@
 #endif
 
 #include "swfmoz_loader.h"
+#include "swfmoz_player.h"
 #include "plugin.h"
 
 /* Note about refcounting:
@@ -47,26 +48,38 @@ swfmoz_loader_dispose (GObject *object)
 }
 
 static void
-swfmoz_loader_load (SwfdecLoader *loader, SwfdecLoader *parent, 
-    SwfdecLoaderRequest request, const char *data, gsize data_len)
+swfmoz_loader_load (SwfdecLoader *loader, SwfdecPlayer *player, 
+    const char *url, SwfdecLoaderRequest request, SwfdecBuffer *buffer)
 {
+  SwfmozPlayer *mozplay = SWFMOZ_PLAYER (player);
   SwfmozLoader *moz = SWFMOZ_LOADER (loader);
-  const char *url;
 
-  moz->instance = SWFMOZ_LOADER (parent)->instance;
-  g_object_ref (moz);
-  url = swfdec_url_get_url (swfdec_loader_get_url (loader));
-  if (request == SWFDEC_LOADER_REQUEST_POST) {
-    plugin_post_url_notify (moz->instance, url, NULL, data, data_len, moz);
+  moz->waiting_for_stream = TRUE;
+  moz->instance = mozplay->instance;
+  if (mozplay->initial) {
+    swfmoz_loader_set_stream (moz, mozplay->initial);
+    mozplay->initial = NULL;
   } else {
-    plugin_get_url_notify (moz->instance, url, NULL, moz);
+    g_object_ref (moz);
+    if (request == SWFDEC_LOADER_REQUEST_POST) {
+      if (buffer) {
+	plugin_post_url_notify (moz->instance, url, NULL, 
+	    (char *) buffer->data, buffer->length, moz);
+      } else {
+	plugin_post_url_notify (moz->instance, url, NULL, NULL, 0, moz);
+      }
+    } else {
+      plugin_get_url_notify (moz->instance, url, NULL, moz);
+    }
   }
 }
 
 static void
-swfmoz_loader_close (SwfdecLoader *loader)
+swfmoz_loader_close (SwfdecStream *stream)
 {
-  SwfmozLoader *moz = SWFMOZ_LOADER (loader);
+  SwfmozLoader *moz = SWFMOZ_LOADER (stream);
+
+  moz->waiting_for_stream = FALSE;
 
   if (moz->stream)
     plugin_destroy_stream (moz->instance, moz->stream);
@@ -76,12 +89,14 @@ static void
 swfmoz_loader_class_init (SwfmozLoaderClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  SwfdecStreamClass *stream_class = SWFDEC_STREAM_CLASS (klass);
   SwfdecLoaderClass *loader_class = SWFDEC_LOADER_CLASS (klass);
 
   object_class->dispose = swfmoz_loader_dispose;
 
+  stream_class->close = swfmoz_loader_close;
+
   loader_class->load = swfmoz_loader_load;
-  loader_class->close = swfmoz_loader_close;
 }
 
 static void
@@ -90,15 +105,22 @@ swfmoz_loader_init (SwfmozLoader *slow_loader)
 }
 
 void
-swfmoz_loader_set_stream (SwfmozLoader *loader, NPP instance, NPStream *stream)
+swfmoz_loader_set_stream (SwfmozLoader *loader, NPStream *stream)
 {
   g_return_if_fail (SWFMOZ_IS_LOADER (loader));
   g_return_if_fail (loader->stream == NULL);
-  g_return_if_fail (instance != NULL);
   g_return_if_fail (stream != NULL);
 
+  if (!loader->waiting_for_stream) {
+    plugin_destroy_stream (loader->instance, stream);
+    return;
+  }
+
+  loader->waiting_for_stream = FALSE;
+
   g_printerr ("Loading stream: %s\n", stream->url);
-  loader->instance = instance;
+  g_object_ref (loader);
+  stream->pdata = loader;
   loader->stream = stream;
   if (stream->end)
     swfdec_loader_set_size (SWFDEC_LOADER (loader), stream->end);
@@ -111,26 +133,12 @@ swfmoz_loader_ensure_open (SwfmozLoader *loader)
 
   if (loader->open)
     return;
-  swfdec_loader_open (SWFDEC_LOADER (loader), loader->stream->url);
+  swfdec_loader_set_url (SWFDEC_LOADER (loader), loader->stream->url);
+  swfmoz_player_add_loader (loader->instance->pdata, loader);
+  swfdec_stream_open (SWFDEC_STREAM (loader));
   loader->open = TRUE;
 }
 
-SwfdecLoader *
-swfmoz_loader_new (NPP instance, NPStream *stream)
-{
-  SwfmozLoader *ret;
-  SwfdecURL *url;
-
-  g_return_val_if_fail (stream != NULL, NULL);
-
-  url = swfdec_url_new (stream->url);
-  ret = g_object_new (SWFMOZ_TYPE_LOADER, "url", url, NULL);
-  swfdec_url_free (url);
-  swfmoz_loader_set_stream (ret, instance, stream);
-
-  return SWFDEC_LOADER (ret);
-}
-
 const char *
 swfmoz_loader_get_data_type_string (SwfdecLoader *loader)
 {
@@ -148,6 +156,10 @@ swfmoz_loader_get_data_type_string (SwfdecLoader *loader)
       return "Text";
     case SWFDEC_LOADER_DATA_XML:
       return "XML Data";
+    case SWFDEC_LOADER_DATA_JPEG:
+      return "JPEG image";
+    case SWFDEC_LOADER_DATA_PNG:
+      return "PNG image";
     default:
       g_printerr ("unknown data type %u\n", 
 	  (guint) swfdec_loader_get_data_type (loader));
